@@ -29,8 +29,12 @@
 
 #include_next <string.h>
 
+#ifndef HAVE_STRLCAT
 extern size_t strlcat(char *dst, const char *src, size_t dstsize);
+#endif
+
+#ifndef HAVE_STRLCPY
 extern size_t strlcpy(char *dst, const char *src, size_t len);
-extern size_t strnlen(const char *str, size_t maxlen);
+#endif
 
 #endif
