@@ -26,7 +26,7 @@
 #include <zone.h>
 
 zoneid_t
-getzoneid()
+getzoneid(void)
 {
 	return (GLOBAL_ZONEID);
 }
