@@ -56,10 +56,10 @@
 #define	ZDIFF_REMOVED	'-'
 #define	ZDIFF_RENAMED	"R"
 
-#define	ZDIFF_ADDED_COLOR ANSI_GREEN
+#define	ZDIFF_ADDED_COLOR    ANSI_GREEN
 #define	ZDIFF_MODIFIED_COLOR ANSI_YELLOW
-#define	ZDIFF_REMOVED_COLOR ANSI_RED
-#define	ZDIFF_RENAMED_COLOR ANSI_BLUE
+#define	ZDIFF_REMOVED_COLOR  ANSI_RED
+#define	ZDIFF_RENAMED_COLOR  ANSI_BOLD_BLUE
 
 /*
  * Given a {dsname, object id}, get the object path
